//+------------------------------------------------------------------+
//|                                                 ONNX_TEST_01.mq5 |
//|                                  Copyright 2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+

#define UNDEFINED_REPLACE 1

int OnInit()
  {
//---
   long onnx_handle = OnnxCreate("best_score_model.onnx",ONNX_COMMON_FOLDER);
    if(onnx_handle == INVALID_HANDLE)
      {
         Print("Failed to load model");
         return(INIT_FAILED);
      }else
         {
            Print("Load model success");
         }
   vector v_input = vector::Ones(49);
   vector v_output(3);
   vector x_output(1);
   //double value;
   Print("v_input: ",v_input);
   Print("v_output: ",v_output);
   Print("x_output: ",x_output);
   
   OnnxTypeInfo type_info;

   long input_count=OnnxGetInputCount(onnx_handle);
   Print("model has ",input_count," input(s)");
   for(long i=0; i<input_count; i++)
     {
      string input_name=OnnxGetInputName(onnx_handle,i);
      Print(i," input name is ",input_name);
      if(OnnxGetInputTypeInfo(onnx_handle,i,type_info))
         PrintTypeInfo(i,"input",type_info);
     }

   long output_count=OnnxGetOutputCount(onnx_handle);
   Print("model has ",output_count," output(s)");
   for(long i=0; i<output_count; i++)
     {
      string output_name=OnnxGetOutputName(onnx_handle,i);
      Print(i," output name is ",output_name);
      if(OnnxGetOutputTypeInfo(onnx_handle,i,type_info))
         PrintTypeInfo(i,"output",type_info);
     }
     
   //-----------------------
   const long input_shape[] = {1,49};
   if(!OnnxSetInputShape(onnx_handle,ONNX_DEFAULT,input_shape))
     {
         Print("OnnxSetInputShape error ",GetLastError());
         return(INIT_FAILED);
     }
   
   const long output_shape[] = {1,3};
   if(!OnnxSetOutputShape(onnx_handle,0,output_shape))
     {
         Print("OnnxSetOutputShape error ",GetLastError());
         return(INIT_FAILED);
     }
     
   const long outval_shape[] = {1,1};
   if(!OnnxSetOutputShape(onnx_handle,1,outval_shape))
     {
         Print("OnnxSetOutvalShape error ",GetLastError());
         return(INIT_FAILED);
     }
   //-----------------------
   
   //--- run the model
   if(!OnnxRun(onnx_handle,ONNX_DEBUG_LOGS | ONNX_NO_CONVERSION,v_input,v_output,x_output))
     {
      Print("OnnxRun failed, error ",GetLastError());
      OnnxRelease(onnx_handle);
      return(INIT_FAILED);
     }else
        {
            Print("v_output: ",v_output);
        }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {
//---
   double ret=0.0;
//---

//---
   return(ret);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| PrintTypeInfo                                                    |
//+------------------------------------------------------------------+
void PrintTypeInfo(const long num,const string layer,const OnnxTypeInfo& type_info)
  {
   Print("   type ",EnumToString(type_info.type));
   Print("   data type ",EnumToString(type_info.element_type));

   if(type_info.dimensions.Size()>0)
     {
      bool   dim_defined=(type_info.dimensions[0]>0);
      string dimensions=IntegerToString(type_info.dimensions[0]);
      for(long n=1; n<type_info.dimensions.Size(); n++)
        {
         if(type_info.dimensions[n]<=0)
            dim_defined=false;
         dimensions+=", ";
         dimensions+=IntegerToString(type_info.dimensions[n]);
        }
      Print("   shape [",dimensions,"]");
      //--- not all dimensions defined
      if(!dim_defined)
         PrintFormat("   %I64d %s shape must be defined explicitly before model inference",num,layer);
      //--- reduce shape
      uint reduced=0;
      long dims[];
      for(long n=0; n<type_info.dimensions.Size(); n++)
        {
         long dimension=type_info.dimensions[n];
         //--- replace undefined dimension
         if(dimension<=0)
            dimension=UNDEFINED_REPLACE;
         //--- 1 can be reduced
         if(dimension>1)
           {
            ArrayResize(dims,reduced+1);
            dims[reduced++]=dimension;
           }
        }
      //--- all dimensions assumed 1
      if(reduced==0)
        {
         ArrayResize(dims,1);
         dims[reduced++]=1;
        }
      //--- shape was reduced
      if(reduced<type_info.dimensions.Size())
        {
         dimensions=IntegerToString(dims[0]);
         for(long n=1; n<dims.Size(); n++)
           {
            dimensions+=", ";
            dimensions+=IntegerToString(dims[n]);
           }
         string sentence="";
         if(!dim_defined)
            sentence=" if undefined dimension set to "+(string)UNDEFINED_REPLACE;
         PrintFormat("   shape of %s data can be reduced to [%s]%s",layer,dimensions,sentence);
        }
     }
   else
      PrintFormat("no dimensions defined for %I64d %s",num,layer);
  }
//+------------------------------------------------------------------+